/* Tennis Tournament Management System 2005 by Silvio Moioli
   and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it). */
#include "file.h"

/**
 * Crea un nuovo file (vuoto).
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da
 * creare, completo di percorso.
 */
void fileCrea(const char* nomeFile){
    ofstream daCreare;
    daCreare.open(nomeFile, ios::app);
    if (daCreare.fail()){
        cout << "Impossibile creare il file. Disco pieno?" << endl;
        exit(1);
    }
    daCreare.close();
}

/**
 * Determina se il file &egrave; vuoto (lunghezza 0 caratteri).
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da
 * controllare, completo di percorso.
 * @return true se il file &egrave; vuoto, false altrimenti
 */
bool fileVuoto(const char* nomeFile){
    ifstream daContare;
    daContare.open(nomeFile);
    daContare.seekg(0, ios::end);
    int lunghezza = daContare.tellg();
    daContare.close();
    return lunghezza==0;
}

/**
 * Controlla se il prossimo campo da aggiungere &egrave; il primo del suo 
 * record.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file da
 * controllare, completo di percorso.
 * @return true se il prossimo campo aggiunto sar&agrave; il primo del suo
 * record, false altrimenti.
 */
bool filePrimoCampo(const char* nomeFile){
    if (fileVuoto(nomeFile)){
        return true;
    }

    ifstream daControllare;
    daControllare.open(nomeFile);
    daControllare.seekg( -1, ios::end);
    char precedente = daControllare.get();
    daControllare.close();
    if (precedente == '\n'){
        return true;
    }

    return false;
}

/**
 * Conta il numero di record all'interno del file (versione interna,
 * presuppone il file gi&agrave; aperto in lettura).
 * 
 * @param file un puntatore a un ifstream gi&agrave; aperto
 * @return il numero di record presenti nel file
 */
int fileContaRecordInt(ifstream* file){
    int record = 0;
    file->seekg(0, ios::beg);
    while (!file->eof()){
        char c = file->get();
        if (c == '\n'){
            record++;
        }
    }

    //Mi tolgo dall'eof e finisco
    file->seekg(0, ios::beg);
    file->clear();
    return record;
}

/**
 * Posiziona lo stream all'inizio dell y-esimo record.
 * Note: le coordinate x ed y funzionano come negli array multidimensionali
 * in C, sono numerate quindi da 0 a n-1, dove n &egrave; il numero
 * degli elementi.
 * Per "campo" si intende una stringa memorizzata in un file CSV.
 * Per "record" si intende una riga di un file CSV, composta da uno o 
 * pi&ugrave; "campi".
 * 
 * @param file un puntatore a un ifstream gi&agrave; aperto
 * @param y l'indice del record
 */
void filePosizionaRecord(ifstream* file, int y){
    file->seekg(0, ios::beg);
    //Salta i record
    for (int i = 0; i < y; i++){
        char c = 'a';
        while ((c != '\n') && (!file->eof())){
            c = file->get();
        }
    }
}

/**
 * Posiziona lo stream all'inizio dell x-esimo campo nell'y-esimo record.
 * Note: le coordinate x ed y funzionano come negli array multidimensionali
 * in C, sono numerate quindi da 0 a n-1, dove n &egrave;
 * il numero degli elementi.
 * Per "campo" si intende una stringa memorizzata in un file CSV.
 * Per "record" si intende una riga di un file CSV, composta da uno o 
 * pi&ugrave; "campi".
 * 
 * @param file un puntatore a un ifstream gi&agrave; aperto
 * @param x l'indice del campo
 * @param y l'indice del record
 * @return il numero di byte dall'inizio del file al primo carattere 
 * dell'x-esimo campo nell'y-esimo record
 */
void filePosizionaCampo(ifstream* file, int x, int y){
    filePosizionaRecord(file, y);
    //Salta i campi
    for (int i = 0; i < x; i++){
        char c = 'a';
        while (c != ','){
            c = file->get();
        }
    }
}

/**
 * Legge e ritorna il numero di caratteri da questa posizione
 * del file alla prossima virgola o fine riga (lunghezza del
 * prossimo campo, se lo stream &egrave; posizionato
 * all'inizio del campo stesso).
 * 
 * @param file un puntatore a un ifstream gi&agrave; aperto
 * @return la lunghezza del prossimo campo (in caratteri)
 */
int lunghezzaProssimoCampo(ifstream* file){
    int risultato = 0;

    char c = file->get();
    while (c != ',' && c != '\n'){
        risultato++;
        c = file->get();
    }

    return risultato;
}

/* Per i commenti sulle funzioni pubbliche e i loro i parametri (che sono 
  stati eliminati per  evitare inutili duplicazioni e spreco di spazio)
  si rimanda all'header  della libreria. */

void fileAggiungiCampo(const char* nomeFile, const char* s){
    if (!fileEsiste(nomeFile)){
        fileCrea(nomeFile);
    }

    bool primo = filePrimoCampo(nomeFile);

    ofstream fileScrivi;
    fileScrivi.open(nomeFile, ios::app);
    if (!primo){
        fileScrivi << ",";
    }
    fileScrivi << s;
    fileScrivi.close();
}

void fileAggiungiCampoUltimoChar(const char* nomeFile, const char* s){
    if (!fileEsiste(nomeFile)){
        fileCrea(nomeFile);
        fileAggiungiCampo(nomeFile, s);
    }
    else {
        filebuf fileScrivi;
        ostream output(&fileScrivi);
        istream input(&fileScrivi);
        fileScrivi.open (nomeFile, ios::in | ios::out);

        output.seekp( -1, ios_base::end);
        output << s;
        fileScrivi.close();
    }
}

void fileNuovoRecord(const char* nomeFile){
    if (!fileEsiste(nomeFile)){
        fileCrea(nomeFile);
    }

    ofstream fileScrivi;
    fileScrivi.open(nomeFile, ios::app);
    fileScrivi << endl;
    fileScrivi.close();
}

int fileContaRecord(const char* nomeFile){
    ifstream file;
    file.open(nomeFile);
    int risultato = fileContaRecordInt(&file);
    file.close();
    return risultato;
}

bool fileEsiste(const char* nomeFile){
    ifstream daControllare;
    daControllare.open(nomeFile);
    if (daControllare.fail()){
        return false;
    }
    else {
        daControllare.close();
        return true;
    }
}

char fileUltimoCarattere(const char* nomeFile){
    ifstream file;
    file.open(nomeFile);
    file.seekg( -1, ios::end);
    char risultato = file.get();
    file.close();
    return risultato;
}

char* fileLeggiCampo(const char* nomeFile, int x, int y){
    ifstream file;
    file.open(nomeFile);

    filePosizionaCampo(&file, x, y);

    /*
     * Vecchia implementazione: inspiegabilmente ha problemi
     * su alcune versioni del gcc su Windows, in particolare
     * con quella del dev-c++ 5.
     * 
     * int puntatorePrecedente = file.tellg(); 
     * int lunghezza = lunghezzaProssimoCampo(&file);
     * char* risultato = new char[lunghezza+1];
     * file.seekg(puntatorePrecedente); 
     */

    //Nuova implementazione: meno efficiente...
    int lunghezza = lunghezzaProssimoCampo(&file);
    char* risultato = new char[lunghezza + 1];
    //...perche' richiama fpc() due volte.
    filePosizionaCampo(&file, x, y);

    for (int i = 0;i < lunghezza;i++){
        risultato[i] = file.get();
    }
    risultato[lunghezza] = '\0';

    file.close();
    return risultato;
}
