/* Tennis Tournament Management System 2005 by Silvio Moioli
   and Davide Gottini (moio@tiscali.it, davide.gottini@virgilio.it). */
/**
 * Liberia giocatore.h
 * Gestisce le operazioni ad alto livello sui giocatori.
 * N.B.: Questa &egrave; l'implementazione "Release", che non 
 * contiene molti controlli  extra (assertions) presenti nella
 * versione "Debug".
 */
#include <cstdlib>
#include <fstream>
#include <iostream>
#include "liste.h"
using namespace std;

#ifndef TT_GIOCATORE
#define TT_GIOCATORE

/**
 * Costante simbolica: indica un giocatore che che pu&ograve; giocare
 *  nel turno corrente.
 */
#define LIBERO 0
/**
 * Costante simbolica: indica un giocatore che &egrave; stato eliminato
 * in un turno precedente.
 */
#define ELIMINATO 1
/**
 * Costante simbolica: indica un giocatore che ha gi&agrave; giocato
 * in questo turno.
 */
#define GIOCATO 2

/**
 * Costante simbolica: indica un giocatore che ha vinto la partita.
 */
#define HAVINTO 3
/**
 * Costante simbolica: indica un giocatore che ha perso la partita.
 */
#define HAPERSO 4
/**
 * Costante simbolica: indica un giocatore che non ha giocato
 * in questa partita.
 */
 #define NONGIOCA 5

/**
 * Struttura che descrive un giocatore del torneo.
 */
struct Giocatore {
    /** Il numero identificativo del giocatore */
    int id;
    /** Il nome del giocatore, terminato da '\\0' */
    char* nome;
};

/**
 * Ritorna una struttura che descrive il giocatore identificato dall'id
 * specificato.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param id l'id del giocatore
 * @return una struttura Giocatore con i dettagli
 */
Giocatore* giocDettagliGiocatore(const char* nomeFile, int id);

/**
 * Ritorna una delle costanti LIBERO, ELIMINATO o GIOCATO che
 * descrivono lo stato di un giocatore nel Torneo. In particolare:
 * <ul>
 * <li>lo stato LIBERO corrisponde a un giocatore che ha la possibilit&agrave;
 * di giocare in questo turno o in turni successivi (non &egrave; mai stato 
 * eliminato e non ha ancora giocato in questo turno);</li>
 * <li>lo stato ELIMINATO indica che il giocatore &egrave; gi&agrave; stato 
 * battuto in un turno precedente a questo;</li>
 * <li>lo stato GIOCATO indica che il giocatore ha preso parte a una partita
 * nel turno corrente.</li>
 * </ul>
 *
 * @param nomeFile stringa terminata da '\\0' con il nome del file delle
 * partite, completo di percorso.
 * @param id il numero identificativo del giocatore
 * @return LIBERO, ELIMINATO o GIOCATO
 */
int giocStatoGiocatore(const char* nomeFile, int id);

/**
 * Ritorna una lista nella quale ogni elemento &egrave; una struttura 
 * giocatore con i dettagli relativi a un giocatore libero.
 * 
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso.
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file 
 * delle partite, completo di percorso.
 * @return una Lista con i dettagli di tutti i giocatori liberi nel Torneo.
 */
Lista* giocGiocatoriLiberi(const char* nomeFileGiocatori,
    const char* nomeFilePartite);

/**
 * Ritorna il nome del giocatore avente l'id specificato.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso.
 * @param idGiocatore id del giocatore
 * @return il nome del giocatore, in formato stringa
 */
char* giocNomeGiocatore(const char* nomeFile, int idGiocatore);

/**
 * Inserisce un nuovo giocatore nel Torneo.
 * 
 * @param nomeFile stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso.
 * @param nomeGiocatore il nome del giocatore da inserire
 */
void giocNuovoGiocatore(const char* nomeFile, char* nomeGiocatore);

/**
 * Fornisce informazioni sul ruolo di un giocatore in una data partita.
 * Ritorna una delle costanti simboliche HAVINTO, HAPERSO o NONGIOCA
 * a seconda se il giocatore risulti vincitore, perdente o non abbia preso
 * parte a una partita.
 * 
 * @param nomeFileGiocatori stringa terminata da ' \\0' con il nome del file 
 * dei giocatori, completo di percorso
 * @param nomeFilePartite stringa terminata da ' \\0' con il nome del file 
 * delle partite, completo di percorso
 * @param idGiocatore il giocatore di cui si vuole conoscere il ruolo
 * @param idPartita la partita da esaminare
 */
int giocHaGiocato(const char* nomeFileGiocatori, const char* nomeFilePartite, 
    int idGiocatore, int idPartita);

#endif
